package bank;

import java.io.Serializable;
import java.util.Comparator;

import bank.exceptions.DuplicateClientException;

/**
 * Class Holder contains information about holders of bank accounts. Each holder
 * is identified by a number. Note that this class is not comparable.
 */
public class Client implements Serializable {

  /** Serial number for serialization. */
  private static final long serialVersionUID = 202010211636L;

  /**
   * NUMBER_COMPARATOR allow comparing holders based on their ids.
   * 
   * @see java.util.Comparator
   */
  public static final Comparator<Client> NUMBER_COMPARATOR = new Comparator<>() {

    /** Compare two holders by id. */
    @Override
    public int compare(Client holder1, Client holder2) {
      return holder1._id - holder2._id;
    }

  }; // NUMBER_COMPARATOR

  /**
   * NAME_COMPARATOR allow comparing holders based on their names.
   * 
   * @see java.util.Comparator
   */
  public static final Comparator<Client> NAME_COMPARATOR = new Comparator<>() {

    /** Compare two holders by name (lexicographically and ignoring case). */
    @Override
    public int compare(Client h1, Client h2) {
      return h1.getName().compareToIgnoreCase(h2.getName());
    }

  }; // NAME_COMPARATOR

  /** The holder's id. */
  private int _id;

  /** The holder's name. */
  private String _name;

  /**
   * Constructor (initializes id and name).
   * 
   * @param id   the holder's id.
   * @param name the holder's name.
   * @throws DuplicateClientException
   */
  public Client(int id, String name) throws DuplicateClientException {
    _id = id;
    _name = name;
  }

  /**
   * Constructor (initializes from array of String).
   * 
   * @param init the holder's parameters.
   * @throws DuplicateClientException
   */
  public Client(String[] init) throws DuplicateClientException {
    this(Integer.parseInt(init[1]), init[2]);
  }

  /**
   * @return the holder's id.
   */
  public final int getId() {
    return _id;
  }

  /**
   * @return the holder's name.
   */
  public String getName() {
    return _name;
  }

  /**
   * @param name the new name.
   */
  public final void setName(String name) {
    _name = name;
  }

  /** @see java.lang.Object#equals(java.lang.Object) */
  @Override
  public boolean equals(Object holder) {
    return holder instanceof Client && ((Client) holder).getId() == _id;
  }

  /** @see java.lang.Object#toString() */
  @Override
  public String toString() {
    return "<holder name='" + getName() + "' id='" + getId() + "'/>";
  }

}
